%**************************************************************************
% File inductance.m
% 
% Used to compare different conductor diameters for coil design.
%
% Written by Zdenko Tudor, 2011
%**************************************************************************

clc;
clear all;

% 10x10 side
% Input parameters
x = 0.075;               % [m] x-length
y = 0.075;               % [m] y-length
Power = 0.25;            % [W] Dissipated power
V = 5;                   % [V] Battery voltage
mu_0 = 4*pi*1e-7;        % Permability

% Possible wire data
% [m] Wire diameter
diameter = [0.455, 0.405, 0.361, 0.321, 0.286, 0.255, 0.227, 0.202,...
            0.18, 0.16, 0.143, 0.127, 0.113, 0.101]/1000;
diameter = fliplr(diameter);

% [Ohm/m] Wire resistance per meter
res = [0.106, 0.134, 0.169, 0.213, 0.268, 0.339, 0.427, 0.538, 0.679,...
       0.856, 1.08, 1.36, 1.72, 2.16];
res = fliplr(res);
   
% [kg/m] Wire weight per meter
weight = [.00144, .00114, .000908, .000720, .000571, .000453, .000359,...
          .000285, .000226, .000179, .000142, .000113, .0000893, .0000797];
weight = fliplr(weight);

% Parameters
A = x*y;            % [m^2] Coil area
Rcoil = V^2/Power;  % [Ohm] Resistance needed for given power dissipation
turnL = 2*(x+y);    % [m] Length of one coil winding

N=zeros(length(res),1);     % Turns
m=zeros(length(res),1);     % Maximum magnetic momentum
w=zeros(length(res),1);     % Total coil weight
L=zeros(length(res),1);     % Inductance
t=zeros(length(res),1);     % Coils time constant L/R
C=zeros(length(res),1);     % Crossection of coil
for j = 1:length(res)
    N(j) = floor(Rcoil/(turnL*res(j)));                       % Turns
    m(j) = V*A*N(j)/Rcoil;                                    % Momentum
    w(j) = N(j)*turnL*weight(j);                              % Weight
    L(j) = N(j)^2*2*mu_0*x*(asinh(x/(diameter(j)/2))-1)/pi;   % Inductance
    t(j) = L(j)/Rcoil;                                        % Tau
    C(j) = diameter(j)^2*N(j)*100^2;                          % Crossection
end

% Print data for k-th diameter
k=8;
P = [diameter(k)*1000;N(k);C(k);w(k);m(k);t(k)];
fprintf('Diameter: %fmm\nTurns: %f\nCrossection: %fcm^2\nWeight:',P(1), P(2), P(3))
fprintf('%fKg\nMaximum magnetic momentum: %fAm^2\nTau: %fs\n',P(4), P(5), P(6));
