%**************************************************************************
% File eul2qua.m
%
% [W_B_IB,QUA] = eul2qua(P,W_B_OB,EUL)
%
% Function used for converting Euler angles to quaternions. This function
% also calculates the angular velocity between B and I represented in B.
%
% Input:
% P - struct containg necessary parameter(s) (w_o)
% W_B_OB - Is W^B_OB, a vector containing the three rotational velocity 
% components. Size 3x1
% EUL - Vector containing Euler angles. Size 3x1. Form: [phi, theta, psi]'.
% Angles are in degrees.
%
% Output:
% W_B_IB - Is W^B_IB, a vector containing the three rotational velocity 
% components. Size 3x1
% QUA - Vector containing quaternions. Size 4x1. 
% Form: [eta eps1 eps2 eps3]'.
%
% Example:
% [w_B_OB,eul] = qua2eul(P,yout(:,1:3),yout(:,4:7));
%
% Written by Per Kolbjrn Soglo, 1994
% Slightly edited by Zdenko Tudor, 2011
%**************************************************************************
function [w_B_IB,qua] = eul2qua(P,w_B_OB,eul)
% Load neccessary parameter
w_o=P.w_o;

% Conversion from degrees to radians
eul(1) = pi*eul(1)/180;
eul(2) = pi*eul(2)/180;
eul(3) = pi*eul(3)/180;

% Rotation matrix from O to B
c1 = cos(eul(1)); c2 = cos(eul(2)); c3 = cos(eul(3));
s1 = sin(eul(1)); s2 = sin(eul(2)); s3 = sin(eul(3));

% XYZ-rotation
R11 = c2*c3;
R12 = -c2*s3;
R13 = s2;
R21 = c1*s3+c3*s1*s2;
R22 = c1*c3-s1*s2*s3;
R23 = -c2*s1;
R31 = s1*s3-c1*c3*s2;
R32 = c1*s2*s3+c3*s1;
R33 = c1*c2;
R_O_B = [R11 R12 R13
        R21 R22 R23
        R31 R32 R33];

% Calculating angular velocity between O and I represented in B
w_B_IO(1) = -w_o*R21;
w_B_IO(2) = -w_o*R22;
w_B_IO(3) = -w_o*R23;

% Calculating angular velocity between B and I represented in B
w_B_IB(1) = w_B_OB(1) + w_B_IO(1);
w_B_IB(2) = w_B_OB(2) + w_B_IO(2);
w_B_IB(3) = w_B_OB(3) + w_B_IO(3);
w_B_IB = w_B_IB';

% Calculating Euler parameters
R44 = trace(R_O_B);
maxVal = max([R11 R22 R33 R44]);
if R11 == maxVal
    p1 = sqrt(1 + 2*R11-R44);
    p2 = (R21 + R12)/p1;
    p3 = (R13 + R31)/p1;
    p4 = (R32 - R23)/p1;
elseif R22 == maxVal
    p2 = sqrt(1 + 2*R22-R44);
    p1 = (R21 + R12)/p2;
    p3 = (R32 + R23)/p2;
    p4 = (R13 - R31)/p2;
elseif R33 == maxVal
    p3 = sqrt(1 + 2*R33-R44);
    p1 = (R13 + R31)/p3;
    p2 = (R32 + R23)/p3;
    p4 = (R21 - R12)/p3;
else
    p4 = sqrt(1 + R44);
    p1 = (R32 - R23)/p4;
    p2 = (R13 - R31)/p4;
    p3 = (R21 - R12)/p4;
end

qua(1) = p4/2;
qua(2) = p1/2;
qua(3) = p2/2;
qua(4) = p3/2;
qua = qua';
end