%**************************************************************************
% File main.m
% Main script. Contains the following sections:
%
% - Simulate (integrate) system
% - Plot angular velocity w^B_OB
% - Plot euler angles
% - Plot quaternions
% - Plot moments generated by the coils
% - Plot power consumption
% - Plot torques generated by the coils
% - Plot angylar velocity w^B_IB
% - Create satellite animation (the aviobject lines should be uncommented)
%
% Written by Zdenko Tudor, 2011
%**************************************************************************

clear all; clc;
addpath('myIGRF');

% Initialize storage variables (For testing and plotting purposes only)
global VAR;

% Create parameters struct
P = parameters();

% Load neccessary parameters
orbitPeriod = P.orbitPeriod;        % For defining simulation length
scalePlot = P.scalePlot;            % For x-axes scaling when plotting

% Initial and simulation parameters
numOfOrbits = 4;                  % Simulation time in number of orbits
tSpan = [0,orbitPeriod*numOfOrbits];% Time span for ODE45

w_B_OB = [0;-0.2;0];                        % Initial rotational velocity
eulAng = [0;0;0];                       % Initial orientation    
[w_B_IB,qua] = eul2qua(P,w_B_OB',eulAng');  % Transform initial parameters
initLat = 0;                                % Initial latitude
initJoule = 0;                              % Always to be set to zero

% State vector x = [wx wy wz eta eps1 eps2 eps3 latitude totalJoule];
xInit = [w_B_IB;qua;initLat;initJoule];

% ODE45 simulaton settings
 options = odeset('MaxStep',5,'OutputFcn',@outFcn,'Refine',1);
% options = odeset('MaxStep',0.2,'OutputFcn',@outFcn,'RelTol',1e-12,'AbsTol',1e-12,'Refine',1);

% Simulate system
[tout,yout] = ode45(@(t,x)nonlinearSatellite(t,x,P),tSpan,xInit,options);

%% Angular velocity w^B_OB
w_B_OB = zeros(length(tout),3);
for i=1:size(tout)
    S_eps = [ 0, -yout(i,7), yout(i,6);
        yout(i,7), 0, -yout(i,5);
        -yout(i,6), yout(i,5), 0];
    R_O_B = eye(3) + 2*yout(i,4)*S_eps + 2*S_eps^2;
    R_B_O = R_O_B';
    
    w_B_OB(i,1:3) = yout(i,1:3)'-R_B_O*[0;-P.w_o;0];
end

figure(3)
subplot(3,1,1); hold on; grid on;
title('Horizontal axes represent time in number of orbits')
ylabel('\omega^B_{OB}, x_{comp}')
plot(tout*scalePlot,w_B_OB(:,1),'k','LineWidth',2)
% axis([0 orbitNr -0.01 0.01])

subplot(3,1,2); hold on; grid on;
ylabel('\omega^B_{OB}, y_{comp}')
plot(tout*scalePlot,w_B_OB(:,2),'k','LineWidth',2)
% axis([0 orbitNr -0.1 0.002])

subplot(3,1,3); hold on; grid on;
ylabel('\omega^B_{OB}, z_{comp}')
plot(tout*scalePlot,w_B_OB(:,3),'k','LineWidth',2)
% axis([0 orbitNr -0.01 0.01])

%% Euler angles
[w_B_OB,eul] = qua2eul(P,yout(:,1:3),yout(:,4:7));

figure(4)
subplot(3,1,1); hold on; grid on;
title('Horizontal axes represent time in number of orbits')
ylabel('Roll (x)');
plot(tout*scalePlot,eul(:,1),'k','LineWidth',2)
% plot(tout*scalePlot,eul(:,1),'g','LineWidth',2)

subplot(3,1,2); hold on; grid on;
ylabel('Pitch (y)');
plot(tout*scalePlot,eul(:,2),'k','LineWidth',2)
% plot(tout*scalePlot,eul(:,2),'g','LineWidth',2)

subplot(3,1,3); hold on; grid on;
ylabel('Yaw (z)');
plot(tout*scalePlot,eul(:,3),'k','LineWidth',2)
% plot(tout*scalePlot,eul(:,3),'g','LineWidth',2)

%% Quaternions
figure(9)
subplot(4,1,1); hold on; grid on;
title('Horizontal axes represent time in number of orbits')
ylabel('\eta');
plot(tout*scalePlot,yout(:,4),'k','LineWidth',2)
axis([0 numOfOrbits -1.1 1.1]);

subplot(4,1,2); hold on; grid on;
ylabel('\epsilon_1');
plot(tout*scalePlot,yout(:,5),'k','LineWidth',2)

subplot(4,1,3); hold on; grid on;
ylabel('\epsilon_2');
plot(tout*scalePlot,yout(:,6),'k','LineWidth',2)

subplot(4,1,4); hold on; grid on;
ylabel('\epsilon_3');
plot(tout*scalePlot,yout(:,7),'k','LineWidth',2)

%% Momentums generated by the coils
figure(6)
subplot(3,1,1); hold on; grid on;
title('Horizontal axes represent time in number of orbits')
ylabel('[Am^2] x^+-face')
plot(VAR.t*scalePlot,VAR.moment(:,1),'k','LineWidth',2)

subplot(3,1,2); hold on; grid on;
ylabel('[Am^2] y^+-face')
plot(VAR.t*scalePlot,VAR.moment(:,2),'k','LineWidth',2)

subplot(3,1,3); hold on; grid on;
ylabel('[Am^2] z^+-face')
plot(VAR.t*scalePlot,VAR.moment(:,3),'k','LineWidth',2)

%% Torques generated by the coils
figure(7)
subplot(3,1,1); hold on; grid on;
title('Horizontal axes represent time in number of orbits')
ylabel('x-axis');
plot(VAR.t*scalePlot,VAR.torque(:,1),'k','LineWidth',2)

subplot(3,1,2); hold on; grid on;
ylabel('y-axis');
plot(VAR.t*scalePlot,VAR.torque(:,2),'k','LineWidth',2)

subplot(3,1,3); hold on; grid on;
ylabel('z-axis');
plot(VAR.t*scalePlot,VAR.torque(:,3),'k','LineWidth',2)

%% Power consumption
figure(4)
subplot(2,1,1); hold on; grid on;
title('Horizontal axes represent time in number of orbits')
ylabel('[W] Watt');
plot(VAR.t*scalePlot,VAR.W,'k','LineWidth',2)

subplot(2,1,2); hold on; grid on;
ylabel('[J] Joules');
plot(tout*scalePlot,yout(:,9),'k','LineWidth',2)

%% Angular velocity w^B_IB
figure(1)
subplot(3,1,1); hold on; grid on;
title('Horizontal axes represent time in number of orbits')
ylabel('\omega^B_{IB}, x_{comp}')
plot(tout*scalePlot,yout(:,1),'k','LineWidth',2)

subplot(3,1,2); hold on; grid on;
ylabel('\omega^B_{IB}, y_{comp}')
plot(tout*scalePlot,yout(:,2),'k','LineWidth',2)

subplot(3,1,3); hold on; grid on;
ylabel('\omega^B_{IB}, z_{comp}')
plot(tout*scalePlot,yout(:,3),'k','LineWidth',2)

%% Kinetic energy in the system
figure(2); hold on; grid on;
kinEnergy = zeros(length(tout),1);
for i=1:size(tout)
    kinEnergy(i) =  0.5*yout(i,1:3)*P.I*yout(i,1:3)';
end

plot(tout*scalePlot,kinEnergy,'k','LineWidth',2)
title('Horizontal axis represent time in number of orbits')
ylabel('[J] Energy')


%% Satellit plot (animation)
clc
satX = P.satX;
satY = P.satY;
satZ = P.satZ;
w_o = P.w_o;

colorSet='kkkbgr';
fig=figure(2);
set(fig,'Position',[500 200 1100 700]);%,'Visible','Off')
% aviobj = avifile('example2.avi', 'compression', 'Cinepak');

orbitPeriod = (2*pi)/(P.w_o); oP = 1/orbitPeriod;
tOld = 0;
w_B_OB_old = w_B_OB;
Wh_old = 0;
secInterval = 0;
uOld = zeros(1,3);
set(gcf,'Renderer','zbuffer') 
sat = subplot(3,3,[2,5]);

axis manual
axis([-1 1 -1 1 -1 1]*8);
x = zeros(6,4);
y = zeros(6,4);
z = zeros(6,4);


for i=1:size(yout(:,3),1)
   if floor(tout(i))>secInterval
       secInterval = floor(tout(i))+20;
       
       eta = yout(i,4);
       eps = yout(i,5:7)';

       % Rotation matrices and variables
       lat = yout(i,8);
       lon = pi;
       delta = 0;
       S_eps = [ 0, -eps(3), eps(2);eps(3), 0, -eps(1);-eps(2), eps(1), 0];
       R_O_E = [-cos(lon)*sin(lat)*cos(delta)-sin(lon)*sin(delta), cos(lon)*sin(lat)*sin(delta)-sin(lon)*cos(delta), -cos(lon)*cos(lat);
           cos(lon)*sin(delta)-sin(lon)*sin(lat)*cos(delta), cos(lon)*cos(delta)+sin(lon)*sin(lat)*sin(delta), -sin(lon)*cos(lat);
           cos(lat)*cos(delta), -cos(lat)*sin(delta), -sin(lat)];
       R_O_B = eye(3) + 2*eta*S_eps + 2*S_eps^2;
       rot = R_O_E'*R_O_B;
       
       % Create sphere and orbit
       [s1,s2,s3] = sphere;
       s4=-3*pi/2:0.1:pi;
       zComp=R_O_E'*[0;0;1];

       % Translate and rotate the satellite
       transl = [[eye(3); zeros(1,3)], [-cos(lat)*6;0;sin(lat)*6;1]];
       for j=1:6
           newXYZ=rot*[satX(j,:);satY(j,:);satZ(j,:)];
           newXYZ=transl*[newXYZ;ones(1,4)];
           newXYZ=newXYZ(1:3,:);
           x(j,:) = newXYZ(1,:);
           y(j,:) = newXYZ(2,:);
           z(j,:) = newXYZ(3,:);
       end
       
       % Draw satellite
       cla(sat);
       sat = subplot(3,3,[2,5]);

       for j=1:6
           h=patch(x(j,:),y(j,:),z(j,:),colorSet(j));
           set(h,'edgecolor','w','FaceAlpha',0.8)
       end
       hold on; grid on;
       % Draw earth
       surf(s1*4,s2*4,s3*4); alpha(0.1);
       % Draw orbit
       plot3(sin(s4)*6,zeros(length(s4),1),cos(s4)*6,'k','LineWidth',1)
       % Draw z_o vector
       quiver3(-cos(lat)*6,0,sin(lat)*6,zComp(1)*1.5,zComp(2)*1.5,zComp(3)*1.5);
       xlabel('x');ylabel('y');zlabel('z')
       if (lat < hysLat)
           str = sprintf('Red - x^+_{face}, Green - y^+_{face}, Blue - z^+_{face}\n Detumbling Controller - Lat:%5.2f / %5.2f, Long:0',lat,hysLat);
           title(str)
       else
           str = sprintf('Red - x^+_{face}, Green - y^+_{face}, Blue - z^+_{face}\n Referance Controller - Lat: %5.2f, Long:0',lat);
           title(str); 
       end
       
       
       % Plot velocities (and calculate)
       S_eps = [ 0, -yout(i,7), yout(i,6);
           yout(i,7), 0, -yout(i,5);
           -yout(i,6), yout(i,5), 0];
       R_O_B = eye(3) + 2*yout(i,4)*S_eps + 2*S_eps^2;
       R_B_O = R_O_B';
       w_B_OB = yout(i,1:3)'-R_B_O*[0;-P.w_o;0];
       subplot(3,3,3); hold on; grid on;
       plot(oP*[tOld tout(i)],[w_B_OB_old(1) w_B_OB(1)],'r','LineWidth',2);
       title('w^B_{OB} around the x_b axis')
       subplot(3,3,6); hold on; grid on;
       plot(oP*[tOld tout(i)],[w_B_OB_old(2) w_B_OB(2)],'r','LineWidth',2);
       title('w^B_{OB} around the y_b axis')
       subplot(3,3,9); hold on; grid on;
       plot(oP*[tOld tout(i)],[w_B_OB_old(3) w_B_OB(3)],'r','LineWidth',2);
       title('w^B_{OB} around the z_b axis')

       % Plot energy
       subplot(3,6,15); hold on; grid on;
       kinEnergyOld = 0.5*w_B_OB_old'*P.I*w_B_OB_old;
       kinEnergy = 0.5*w_B_OB'*P.I*w_B_OB;
       plot(oP*[tOld tout(i)],[kinEnergyOld kinEnergy],'r','LineWidth',2);
       ylabel('Kinetic orbit relative energy')
       % Plot watt*hours
       subplot(3,6,16); hold on; grid on;
       plot(oP*[tOld tout(i)],[Wh_old, yout(i,9)],'r','LineWidth',2)
       title('Total joules used')
       
       % Plot moments genereted by coils
       if lat >= u(1,4)
           tmp = 1;
           while lat >= u(tmp,4)
               tmp = tmp+1;
           end
           uNew = u(tmp,1:3);
           u = u(tmp+1:end,:);
       else
           uNew = uOld;
       end
       subplot(3,3,1); hold on; grid on;
       plot([tOld tout(i)], [uOld(1) uNew(1)],'r','LineWidth',2)
       a = get(gcf,'CurrentAxes'); set(a,'XTickLabel','');
       title('x_{coil} torque')
       subplot(3,3,4); hold on; grid on;
       plot([tOld tout(i)], [uOld(2) uNew(2)],'r','LineWidth',2)
       a = get(gcf,'CurrentAxes'); set(a,'XTickLabel','');
       title('y_{coil} torque')
       subplot(3,3,7); hold on; grid on;
       plot([tOld tout(i)], [uOld(3) uNew(3)],'r','LineWidth',2)
       a = get(gcf,'CurrentAxes'); set(a,'XTickLabel','');
       title('z_{coil} torque')

       % Store current datapoints as old datapoints
       tOld = tout(i);
       w_B_OB_old = w_B_OB;
       Wh_old = yout(i,9);
       uOld = uNew;
       drawnow();
%        aviobj = addframe(aviobj,getframe(gcf));
%        pause(1)
   end
end
% close(fig);
% aviobj = close(aviobj);



