%**************************************************************************
% File nonlinearSatellite.m
%
% xDot = nonlinearSatellite(T,X,P)
% 
% Simulates the earth-satellite system. To be used with an ordinary
% differential equation solver such as MATLAB's Runge Kutta(4,5) method,
% ODE45. 
% The input:
% T - time vector (implicit)
% X - state vector, initial state has to be provided. The state vector is
% of the form X = [W^B_IB' Q' LAMBDA JOULE]. W^B_IB is a 3x1 vector
% containing the three rotational velocity components. Q is the quaternion
% vector of the form Q = [eta eps1 eps2 eps3]. LAMBDA is the latitude of
% the satellite and JOULE is only a testing paramemter which will return
% the total Joule consumption of the satellite during the simulation.
% Initial JOULE should always be set to zero.
% P - struct containg necessary parameters (w_o, I, Rc, coil data, voltage
% and current ratings).
%
% Example:
% [TOUT,YOUT] = ode45(@(t,x)nonlinearSatellite(t,x,P),tSpan,xInit,options)
% this produces:
% TOUT - time vector for the integration
% YOUT - Integrated xDot vector for each timestep size(YOUT)=length(TOUT)x9
%
% Written by Zdenko Tudor, 2011
%**************************************************************************

function xDot = nonlinearSatellite(t,x,P)
global R_B_O;

% Load necessary parameters from P struct.
w_o = P.w_o;
I = P.I;

% Normalization of quartenions
x(4:7) = x(4:7)./norm(x(4:7));

% State vector x = [];
w_B_IB = x(1:3);            % Angular velocity vector
eta = x(4);                 % Euler parameter eta
eps = x(5:7);               % Euler parameter epsilon
lat = x(8)                 % Lambda

% Epsilon crossmatrix, skew-symmetric
S_eps = [ 0, -eps(3), eps(2);   
          eps(3), 0, -eps(1);
         -eps(2), eps(1), 0];

% Rotation matrices
R_O_B = eye(3) + 2*eta*S_eps + 2*S_eps^2;
R_B_O = R_O_B';

% Angular velocities of frames relative each other
w_O_IO = [0;-w_o;0];
w_B_OB = w_B_IB-R_B_O*w_O_IO;

% Switch controller after this many orbits
orbitSwitch = 1;

% Torque from controllers. Logic 'if' is for controller switching.
if t<P.orbitPeriod*orbitSwitch*1000
    % Pick detumbling controller
    [tau_m,j] = detumblingController1(P,w_B_OB,lat);
%     [tau_m,j] = detumblingController2(P,w_B_OB,lat);
else
    [tau_m,j] = referenceController(P,w_B_OB,lat,eps);
end

%-----Storing variables used for testing purposes only-----Start-----
global tmpVAR;
tmpVAR.torque = tau_m;
xDot(9,1) = j;
%-----Storing variables used for testing purposes only------End------

% Gravitational torque
c3 = R_B_O(:,3);
tau_g = 3*w_o^2*cross(c3,I*c3);

% Angular acceleration
wDot_B_IB=I\(tau_m+tau_g-cross(w_B_IB,I*w_B_IB));

% Eta
etaDot = -0.5*eps'*w_B_OB;

% Epsilon
epsDot = 0.5*(eta*eye(3)+S_eps)*w_B_OB;

% Variable assignment
xDot(1:3,1) = wDot_B_IB;
xDot(4,1) = etaDot;
xDot(5:7,1) = epsDot;
xDot(8,1) = w_o;
end
 
% Dissipative non-linear detumbling controller.
% Dissipates the kinetic energy from the satellite until w^B_OB == [0 0 0].
function [tau_m,j] = detumblingController1(P,w_B_OB,lat)
global R_B_O;
% Geomagnetic field in Orbit frame and in Body frame, pick diple or IGRF.
% [B_O,~] = dipoleField(P,lat);
[B_O] = IGRF(10,10,lat,0,P);
B_B = R_B_O*B_O;

% Controller gain
d = 4e-5;

% Moment set up by coils before scaling (saturating current)
m_B = -(d/norm(B_B,2)^2)*cross(B_B,w_B_OB);

% Moment set up by coils after scaling
[m_B,j] = currentScaling(P,m_B);

% Torque set up by coils
tau_m = cross(m_B,B_B);
end

% Bdot detumbling controller
% Controls the satellite to follow the geomagnetic field
function [tau_m,j] = detumblingController2(P,w_B_OB,lat)
global R_B_O;
% Geomagnetic field in Orbit frame and in Body frame
[B_O,Bdot_O] = dipoleField(P,lat);
B_B = R_B_O*B_O;
S_w = [0 -w_B_OB(3) w_B_OB(2);
       w_B_OB(3) 0 -w_B_OB(1);
       -w_B_OB(2) w_B_OB(1) 0];
Bdot_B = -S_w*R_B_O*B_O+R_B_O*Bdot_O;

% Controller gain
k = 4e-5;

% Moment set up by coils before scaling (saturating current)
m_B = (-k/norm(B_B,2)^2)*Bdot_B;

% Moment set up by coils after scaling
[m_B,j] = currentScaling(P,m_B);

% Torque set up by coils
tau_m = cross(m_B,B_B);
end

% Satellite reference controller for large deviations from R_B_O = eye(3).
function [tau_m,j] = referenceController(P,w_B_OB,lat,eps)
global R_B_O;
% Geomagnetic field in Orbit frame and in Body frame
% [B_O,~] = dipoleField(P,lat);
[B_O] = IGRF(10,10,lat,0,P);
B_B = R_B_O*B_O;

% Controller gains
d = 4e-5;
k = 5e-8;

% Moment set up by coils before scaling (saturating current)
m_B = (1/norm(B_B,2)^2)*(-d*cross(B_B,w_B_OB)-k*cross(B_B,eps));

% Moment set up by coils after scaling
[m_B,j] = currentScaling(P,m_B);

% Torque set up by coils
tau_m = cross(m_B,B_B);
end

% Scales (saturates) the power consumption if the maximum currents are
% exceeded
function [m_B,j] = currentScaling(P,m_B)
% Load coil parameters from parameter struct
Nx = P.Nx; Ny = P.Ny; Nz = P.Nz;
Ax = P.Ax; Ay = P.Ay; Az = P.Az;
ix_max = P.ix_max; iy_max = P.iy_max; iz_max = P.iz_max;
V = P.V;


% Current scaling
% Creates ratio variable containing a coil's: max current/wanted current.
% The lowest value in ratio variable, if below 1 is the highest current
% violation, thus all the currents should be scaled by this factor.
ix = m_B(1)/(Nx*Ax); ratio(1)=ix_max/abs(ix);
iy = m_B(2)/(Ny*Ay); ratio(2)=iy_max/abs(iy);
iz = m_B(3)/(Nz*Az); ratio(3)=iz_max/abs(iz);

if min(ratio)<1
    m_B = m_B*min(ratio);
end

% Power consumption (Joules)
Wx = abs(V*m_B(1)/(Nx*Ax));
Wy = abs(V*m_B(2)/(Ny*Ay));
Wz = abs(V*m_B(3)/(Nz*Az));
j = Wx+Wy+Wz;

%-----Storing variables used for testing purposes only-----Start-----
global tmpVAR;
tmpVAR.moment=m_B;
tmpVAR.W = Wx+Wy+Wz;
%-----Storing variables used for testing purposes only------End------
end

% Returns the dipole model of the geomagnetic field in Teslas, represented
% in the Orbit frame.s
function [B_O,Bdot_O] = dipoleField(P,lat)
w_o = P.w_o;
Rc = P.Rc;

my_m = 1e17/(4*pi);
B0 = my_m/Rc^3;

Bn = B0*cos(lat);       % x-axis in orbital frame, parallel to the velocity vector
Br = 2*B0*sin(lat);     % z-axis in orbital frame, toward Earth center
B_O = [Bn; 0; Br];

BnDot = -w_o*B0*sin(lat);
BrDot = w_o*2*B0*cos(lat);
Bdot_O = [BnDot; 0; BrDot];
end
