%**************************************************************************
% File parameters.m
%
% P = parameters()
%
% Creates a parameter struct P, which is used to pass the parameters
% between different functions.
%
% Written by Zdenko Tudor, 2011
%**************************************************************************

function P = parameters()
% Coil data
% Battery voltage
V = 5;

% Number of coil windings (turns)
Nx = 355;
Ny = 355;
Nz = 800;

% [m^2] Coil area
Ax = 0.08*0.18;
Ay = 0.08*0.18;
Az = 0.08^2;

% [Ohm] Coil resistance
Rx = 110;
Ry = 110;
Rz = 110;

% [A] Maximum current through coils
ix_max = V/Rx;
iy_max = V/Ry;
iz_max = V/Rz;

% Define satelite animation object
satX=[0 1 1 0;0 1 1 0;0 0 0 0;0 1 1 0; 0 1 1 0;1 1 1 1]; 
satY=[0 0 1 1;0 0 0 0;0 1 1 0;0 0 1 1; 1 1 1 1;0 1 1 0];
satZ=[0 0 0 0;0 0 1 1;0 0 1 1;1 1 1 1; 0 0 1 1;0 0 1 1]*2;

% Move mass center to reference frame center
satX = satX-ones(6,4)./2;
satY = satY-ones(6,4)./2;
satZ = satZ-ones(6,4);

m = 2;                      % [kg] Satellite mass
dx = 0.11;                  % X-axis length
dy = 0.1;                   % Y-axis length
dz = 0.2;                   % Z-axis length
Re = 6371.2e3;              % [m] Earth radius
Rs = 600e3;                 % [m] Satellite altitude
Rc = Re+Rs;                 % [m] Distance from earth center to satellite
Ix = (m/12)*(dy^2+dz^2);    % X-axis inertia
Iy = (m/12)*(dx^2+dz^2);    % Y-axis inertia
Iz = (m/12)*(dx^2+dy^2);    % Z-axis inertia

I = diag([Ix Iy Iz]);       % Inertia matrix

G = 6.67428e-11;            % Earth gravitational constant
M = 5.972e24;               % Earth mass

w_o = sqrt(G*M/Rc^3);       % Satellite angular velocity relative Earth

orbitPeriod = (2*pi)/(w_o);     	% For defining simulation length
scalePlot = 1/orbitPeriod;       	% For x-axes scaling when plotting

% Store variables
P.satX = satX; P.satY = satY; P.satZ = satZ;
P.w_o = w_o;
P.V = V; P.I = I;
P.Nx = Nx; P.Ny = Ny; P.Nz = Nz;
P.Ax = Ax; P.Ay = Ay; P.Az = Az;
P.Rx = Rx; P.Ry = Ry; P.Rz = Rz;
P.ix_max = ix_max; P.iy_max = iy_max; P.iz_max = iz_max;
P.Re = Re; P.Rc = Rc;
P.orbitPeriod = orbitPeriod; P.scalePlot = scalePlot;
end