%**************************************************************************
% File qua2eul.m
%
% [W_B_OB,EUL] = qua2eul(P,W_B_IB,QUA)
%
% Function used for converting quaterions to Euler angles. This function
% also calculates the angular velocity between B and O represented in B.
%
% Input:
% P - struct containg necessary parameter(s) (w_o)
% W_B_IB - Is W^B_IB, a matrix containing the three rotational velocity 
% components for all simulation timesteps. Size Nx3
% QUA - Matrix containing quaternions for all simlation timesteps. 
% Size Nx4. Form: [eta eps1 eps2 eps3].
%
% Output:
% W_B_OB - Is W^B_OB, a matrix containing the three rotational velocity 
% components for all simulation timesteps. Size Nx3
% EUL - Matrix containing Euler angles for all simulation timesteps.
% Size Nx3. Form: [phi, theta, psi].
% Angles are in degrees.
%
% Example:
% [w_B_OB,eul] = qua2eul(P,yout(:,1:3),yout(:,4:7));
%
% Written by Per Kolbjrn Soglo, 1994
% Slightly edited by Zdenko Tudor, 2011
%**************************************************************************

function [w_B_OB,eul] = qua2eul(P,w_B_IB,qua)
% Create angulary velocity vector
w_o=P.w_o;
w_O_IO = [0;-w_o;0];

% Calculating number of vectors
n = size(qua,1);

% Storing inital angle in radians
psi_gammel = 0*pi/180;

% Initiate storing matrices
w_B_OB = zeros(n,3);
eul = zeros(n,3);
for i=1:n,
    % Rotation matrix from O to B
    eta = qua(i,1);
    e1 = qua(i,2);
    e2 = qua(i,3);
    e3 = qua(i,4);
    R11 = eta^2+e1^2-e2^2-e3^2;
    R12 = 2*(e1*e2-eta*e3);
    R13 = 2*(e1*e3+eta*e2);
    R21 = 2*(e1*e2+eta*e3);
    R22 = eta^2-e1^2+e2^2-e3^2;
    R23 = 2*(e2*e3-eta*e1);
    R31 = 2*(e1*e3-eta*e2);
    R32 = 2*(e2*e3+eta*e1);
    R33 = eta^2-e1^2-e2^2+e3^2;
    R_O_B = [R11 R12 R13
             R21 R22 R23
             R31 R32 R33];
    R_B_O = R_O_B';
    
    % Calculating angular velocity between B and O represented in B
    w_B_OB(i,:) = w_B_IB(i,:)' - R_B_O*w_O_IO;
    
    % Calculating Euler angles
    psi = atan2(R21,R11);
    if psi < 0
        psi = psi+2*pi;
    end
    psi_test = pi+psi;
    if psi_test > 2*pi
        psi_test = psi_test-2*pi;
    end
    % Checking for discontinuoities in psi
    d = min([abs(psi_gammel-psi), abs(psi_gammel-2*pi-psi), abs(psi_gammel+2*pi-psi)]);
    d_test = min([abs(psi_gammel-psi_test), abs(psi_gammel-2*pi-psi_test), abs(psi_gammel+2*pi-psi_test)]);
    if d_test < d
        psi = psi_test;
    end
    
    theta = atan2(-R31, cos(psi)*R11 + sin(psi)*R21);
    phi = atan2(sin(psi)*R13 - cos(psi)*R23, -sin(psi)*R12 + cos(psi)*R22);
    psi_gammel = psi;
    if psi > pi
        psi = psi - 2*pi;
    end

    eul(i,1) = phi*180/pi;
    eul(i,2) = theta*180/pi;
    eul(i,3) = psi*180/pi;
end

% Zero whole vector to remove unrealistic results from numerical errors
% tol = 1e-13;
% if max(abs(eul(:,1))) < tol;
%     eul(:,1) = 0;
% end
% if max(abs(eul(:,2))) < tol;
%     eul(:,2) = 0;
% end
% if max(abs(eul(:,3))) < tol;
%     eul(:,3) = 0;
% end
end